'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _uniq2 = require('lodash/uniq');

var _uniq3 = _interopRequireDefault(_uniq2);

var _pick2 = require('lodash/pick');

var _pick3 = _interopRequireDefault(_pick2);

var _find2 = require('lodash/find');

var _find3 = _interopRequireDefault(_find2);

var _sparkCore = require('@ciscospark/spark-core');

var _querystring = require('querystring');

var _querystring2 = _interopRequireDefault(_querystring);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*!
 * Copyright (c) 2015-2017 Cisco Systems, Inc. See LICENSE file.
 */

var Team = _sparkCore.SparkPlugin.extend({
  namespace: 'Team',

  /**
   * Move an existing group conversation into a team.
   * @param {TeamObject} team
   * @param {ConversationObject} conversation
   * @param {Object} activity Reference to the activity that will eventually be
   * posted. Use this to (a) pass in e.g. clientTempId and (b) render a
   * provisional activity
   * @returns {Promise} Resolves with the add activity
   */
  addConversation: function addConversation(team, conversation, activity) {
    var _this = this;

    return this._ensureGeneralConversation(team).then(function (generalConversation) {
      return _this.spark.internal.conversation.prepare(activity, {
        verb: 'add',
        target: _this.spark.internal.conversation.prepareConversation(generalConversation),
        object: {
          id: conversation.id,
          objectType: 'conversation'
        },
        kmsMessage: {
          method: 'create',
          uri: '/authorizations',
          resourceUri: conversation.kmsResourceObjectUrl,
          userIds: [generalConversation.kmsResourceObjectUrl]
        }
      });
    }).then(function (a) {
      return _this.spark.internal.conversation.submit(a);
    });
  },


  /**
   * Add a member to a team
   * @param {TeamObject} team
   * @param {Object} participant
   * @param {Object} activity Reference to the activity that will eventually be
   * posted. Use this to (a) pass in e.g. clientTempId and (b) render a
   * provisional activity
   * @returns {Promise} Resolves with activity that was posted
   */
  addMember: function addMember(team, participant, activity) {
    var _this2 = this;

    return this._ensureGeneralConversation(team).then(function (generalConversation) {
      return _this2.spark.internal.conversation.add(generalConversation, participant, activity);
    });
  },


  /**
   * Create a team
   * @param {NewTeamObject} params
   * @param {string} params.displayName
   * @param {string} params.summary
   * @param {Array} params.participants
   * @returns {Promise} Resolves with the created team
   */
  create: function create(params) {
    var _this3 = this;

    if (!params.displayName) {
      return _promise2.default.reject(new Error('`params.displayName` is required'));
    }

    if (!params.participants || params.participants.length === 0) {
      return _promise2.default.reject(new Error('`params.participants` is required'));
    }

    return _promise2.default.all(params.participants.map(function (participant) {
      return _this3.spark.internal.user.asUUID(participant, { create: true });
    })).then(function (participants) {
      participants.unshift(_this3.spark.internal.device.userId);
      params.participants = (0, _uniq3.default)(participants);
    }).then(function () {
      return _this3._prepareTeam(params);
    }).then(function (payload) {
      return _this3.spark.request({
        method: 'POST',
        service: 'conversation',
        resource: 'teams',
        body: payload
      });
    }).then(function (res) {
      return res.body;
    });
  },


  /**
   * Create a conversation within a team. Currently does not support
   * activities besides add (ie no post or share activities).
   * @param {TeamObject} team
   * @param {NewConversationObject} params
   * @param {string} params.displayName
   * @param {string} params.summary
   * @param {Array} params.participants
   * @param {Object} options
   * @param {boolean} options.includeAllTeamMembers
   * @returns {Promise} Resolves with the newly created conversation
   */
  createConversation: function createConversation(team, params, options) {
    var _this4 = this;

    options = options || {};

    if (!params.displayName) {
      return _promise2.default.reject(new Error('`params.displayName` is required'));
    }

    return this._inferTeamUrl(team).then(function () {
      return _this4._ensureGeneralConversation(team);
    }).then(function (generalConversation) {
      return _promise2.default.all(params.participants.map(function (participant) {
        return _this4.spark.internal.user.asUUID(participant, { create: true });
      })).then(function (participants) {
        participants.unshift(_this4.spark.internal.device.userId);
        params.participants = (0, _uniq3.default)(participants);
      }).then(function () {
        return _this4._prepareTeamConversation(generalConversation, params);
      });
    }).then(function (payload) {
      return _this4.spark.request({
        method: 'POST',
        uri: team.url + '/conversations',
        resource: 'teams',
        qs: (0, _pick3.default)(options, 'includeAllTeamMembers'),
        body: payload
      });
    }).then(function (res) {
      return res.body;
    });
  },


  /**
   * Retrieve a single team
   * @param {Object|Team~TeamObject} team
   * @param {string} team.id
   * @param {Object} options
   * @param {boolean} options.includeTeamConversations
   * @param {boolean} options.includeTeamMembers
   * @returns {Promise} Resolves with the requested team
   */
  get: function get(team, options) {
    var _this5 = this;

    options = options || {};

    var params = (0, _assign2.default)({
      includeTeamConversations: false,
      includeTeamMembers: false
    }, options);

    return this._inferTeamUrl(team).then(function () {
      return _this5.spark.request({
        uri: team.url,
        qs: params
      });
    }).then(function (res) {
      return _this5._recordUUIDs(res.body).then(function () {
        return res.body;
      });
    });
  },


  /**
   * Get the list of conversations for a particular team
   * @param {TeamObject} team
   * @returns {Promise} Resolves with an array of conversations
   */
  listConversations: function listConversations(team) {
    var _this6 = this;

    return this._inferTeamUrl(team).then(function () {
      return _this6.spark.request({
        uri: team.url + '/conversations'
      });
    }).then(function (res) {
      return res.body.items;
    });
  },


  /**
   * Join a team conversation
   * @param {TeamObject} team
   * @param {ConversationObject} conversation
   * @returns {Promise} Resolves with the conversation
   */
  joinConversation: function joinConversation(team, conversation) {
    return this.spark.request({
      method: 'POST',
      uri: team.url + '/conversations/' + conversation.id + '/participants'
    }).then(function (res) {
      return res.body;
    });
  },


  /**
   * Retrieve all teams
   * @param {Object} options
   * @param {boolean} options.includeTeamConversations
   * @param {boolean} options.includeTeamMembers
   * @returns {Promise} Resolves with the requested teams
   */
  list: function list(options) {
    var _this7 = this;

    options = options || {};

    var params = (0, _assign2.default)({
      includeTeamConversations: false,
      includeTeamMembers: false
    }, options);

    return this.spark.request({
      api: 'conversation',
      resource: 'teams',
      qs: params
    }).then(function (res) {
      return _promise2.default.all(res.body.items.map(function (team) {
        return _this7._recordUUIDs(team);
      })).then(function () {
        return res.body.items;
      });
    });
  },


  /**
   * Remove a member from a team
   * @param {TeamObject} team
   * @param {Object} participant
   * @param {Object} activity Reference to the activity that will eventually be
   * posted. Use this to (a) pass in e.g. clientTempId and (b) render a
   * provisional activity
   * @returns {Promise} Resolves with activity that was posted
   */
  removeMember: function removeMember(team, participant, activity) {
    var _this8 = this;

    return this._ensureGeneralConversation(team).then(function (generalConversation) {
      return _this8.spark.internal.conversation.leave(generalConversation, participant, activity);
    });
  },


  /**
   * Remove a team conversation from a team.
   * @param {TeamObject} team
   * @param {ConversationObject} conversation to be removed
   * @param {Object} activity Reference to the activity that will eventually be
   * posted. Use this to (a) pass in e.g. clientTempId and (b) render a
   * provisional activity
   * @returns {Promise} Resolves with the leave activity
   */
  removeConversation: function removeConversation(team, conversation, activity) {
    var _this9 = this;

    return this._ensureGeneralConversation(team).then(function (generalConversation) {
      var properties = {
        verb: 'remove',
        object: (0, _pick3.default)(conversation, 'id', 'url', 'objectType'),
        target: (0, _pick3.default)(generalConversation, 'id', 'url', 'objectType'),
        kmsMessage: {
          method: 'delete',
          uri: '<KRO>/authorizations?' + _querystring2.default.stringify({ authId: conversation.id })
        }
      };

      return _this9.spark.internal.conversation.prepare(activity, properties);
    }).then(function (a) {
      return _this9.spark.internal.conversation.submit(a);
    });
  },


  /**
   * Update the displayName, summary, or teamColor field for a team.
   * @param {TeamObject} team to be updated
   * @param {Object} object with updated displayName, summary, and/or teamColor
   * @param {Object} activity Reference to the activity that will eventually be
   * posted. Use this to (a) pass in e.g. clientTempId and (b) render a
   * provisional activity
   * @returns {Promise} Resolves with posted activity
   */
  update: function update(team, object, activity) {
    return this.spark.internal.conversation.update(team, object, activity);
  },
  _ensureGeneralConversation: function _ensureGeneralConversation(team) {
    if (!team.generalConversationUuid) {
      return _promise2.default.reject(new Error('`team.generalConversationUuid` must be present'));
    }

    if (team.conversations.items.length) {
      var generalConversation = (0, _find3.default)(team.conversations.items, { id: team.generalConversationUuid });

      if (generalConversation) {
        return _promise2.default.resolve(generalConversation);
      }
    }

    return this.spark.internal.conversation.get({ id: team.generalConversationUuid });
  },


  /* eslint-disable require-jsdoc */
  _inferTeamUrl: function _inferTeamUrl(team) {
    var _this10 = this;

    if (team.id) {
      return this.spark.internal.feature.getFeature('developer', 'web-high-availability').then(function (haMessagingEnabled) {
        if (haMessagingEnabled) {
          // recompute team URL each time as the host may have changed since last usage
          return _this10.spark.internal.device.getServiceUrl('conversation').then(function (url) {
            team.url = url + '/teams/' + team.id;

            return team;
          });
        }
        if (!team.url) {
          return _this10.spark.internal.device.getServiceUrl('conversation').then(function (url) {
            team.url = url + '/teams/' + team.id;
            /* istanbul ignore else */
            if (process.env.NODE_ENV !== 'production') {
              _this10.logger.warn('team: inferred team url from conversation id; please pass whole team objects to Team methods');
            }

            return team;
          });
        }

        return _promise2.default.resolve(team);
      });
    }

    return _promise2.default.resolve(team);
  },

  /* eslint-enable require-jsdoc */

  _prepareTeam: function _prepareTeam(params) {
    var payload = this.spark.internal.conversation._prepareConversationForCreation(params);

    payload.objectType = 'team';

    if (params.summary) {
      payload.summary = params.summary;
    }

    return _promise2.default.resolve(payload);
  },

  _prepareTeamConversation: function _prepareTeamConversation(teamConversation, params) {
    if (!teamConversation.kmsResourceObjectUrl) {
      return _promise2.default.reject(new Error('Team general conversation must have a KRO'));
    }

    var payload = this.spark.internal.conversation._prepareConversationForCreation(params);

    // Push the general conversation KRO on so that team members can
    // decrypt the title of this open room without joining it.
    payload.kmsMessage.userIds.push(teamConversation.kmsResourceObjectUrl);

    return _promise2.default.resolve(payload);
  },
  _recordUUIDs: function _recordUUIDs(team) {
    var _this11 = this;

    if (!team.teamMembers || !team.teamMembers.items) {
      return _promise2.default.resolve(team);
    }

    return _promise2.default.all(team.teamMembers.items.map(function (participant) {
      return _this11.spark.internal.user.recordUUID(participant);
    }));
  },
  version: '1.52.7'
});

/**
 * Assign or unassign a team member to be a moderator of a team
 * @param {TeamObject} team
 * @param {Object} participant
 * @param {Object} activity
 * @returns {Promise} Resolves with activity that was posted
 */
['assignModerator', 'unassignModerator'].forEach(function (verb) {
  Team.prototype[verb] = function submitModerationChangeActivity(team, member, activity) {
    var _this12 = this;

    return this._ensureGeneralConversation(team).then(function (teamConversation) {
      return _this12.spark.internal.conversation[verb](teamConversation, member, activity);
    });
  };
});

/**
 * Archive or unarchive a team or team conversation.
 * @param {TeamObject|ConversationObject} target team or team conversation that should be archived
 * @param {Object} activity Reference to the activity that will eventually be
 * posted. Use this to (a) pass in e.g. clientTempId and (b) render a
 * provisional activity
 * @returns {Promise} Resolves with the posted activity
 */
['archive', 'unarchive'].forEach(function (verb) {
  Team.prototype[verb] = function submitArchiveStateChangeActivity(target, activity) {
    var _this13 = this;

    if (!target.objectType) {
      return _promise2.default.reject(new Error('`target.objectType` is required'));
    }

    var properties = {
      verb: verb,
      object: (0, _pick3.default)(target, 'id', 'url', 'objectType'),
      target: (0, _pick3.default)(target, 'id', 'url', 'objectType')
    };

    return this.spark.internal.conversation.prepare(activity, properties).then(function (a) {
      return _this13.spark.internal.conversation.submit(a);
    });
  };
});

exports.default = Team;
//# sourceMappingURL=team.js.map
