/*!
 * Copyright (c) 2015-2017 Cisco Systems, Inc. See LICENSE file.
 */

/* eslint-disable no-underscore-dangle */
import {assert} from '@ciscospark/test-helper-chai';
import MockSpark from '@ciscospark/test-helper-mock-spark';
import sinon from '@ciscospark/test-helper-sinon';
import Team from '@ciscospark/internal-plugin-team';
import User from '@ciscospark/internal-plugin-user';

describe('plugin-team', () => {
  describe('Team', () => {
    let spark;

    const convoUrl = 'https://conv-test.wbx2.com/conversation';

    beforeEach(() => {
      spark = new MockSpark({
        children: {
          team: Team,
          user: User
        }
      });

      spark.internal.user.recordUUID = sinon.spy();
      spark.internal.device.getServiceUrl = sinon.stub().returns(Promise.resolve(convoUrl));
    });

    describe('#create()', () => {
      it('requires a displayName', () => assert.isRejected(spark.internal.team.create({}), /`params.displayName` is required/));

      it('requires a participants attribute', () => assert.isRejected(spark.internal.team.create({displayName: 'test'}), /`params.participants` is required/));

      it('requires a participants array', () => assert.isRejected(spark.internal.team.create({displayName: 'test', participants: []}), /`params.participants` is required/));
    });

    describe('#createConversation()', () => {
      it('requires a displayName', () => assert.isRejected(spark.internal.team.createConversation({}, {}), /`params.displayName` is required/));

      it('requires a team object with a general conversation', () => assert.isRejected(spark.internal.team.createConversation({}, {displayName: 'test'}), /`team.generalConversationUuid` must be present/));
    });

    describe('#prepareTeamConversation()', () => {
      it('requires a KRO', () => assert.isRejected(spark.internal.team._prepareTeamConversation({}), /Team general conversation must have a KRO/));
    });

    describe('#recordUUIDs', () => {
      it('resolves if there are no teamMembers', () => spark.internal.team._recordUUIDs({})
        .then(() => assert.equal(spark.internal.user.recordUUID.callCount, 0)));

      it('resolves if there isn\'t teamMembers.items', () => spark.internal.team._recordUUIDs({teamMembers: {}})
        .then(() => assert.equal(spark.internal.user.recordUUID.callCount, 0)));
    });

    describe('#_inferTeamUrl', () => {
      const testTeam = {test: 'team'};

      it('Returns given team if no id', () => spark.internal.team._inferTeamUrl(testTeam)
        .then((team) => {
          assert.notCalled(spark.internal.feature.getFeature);
          assert.notCalled(spark.internal.device.getServiceUrl);
          assert.equal(team.test, 'team');
        }));
      describe('HA is disabled', () => {
        beforeEach(() => {
          spark.internal.feature.getFeature = sinon.stub().returns(Promise.resolve(false));
          testTeam.id = 'id1';
        });
        it('returns unmodified team if URL is defined', () => {
          testTeam.url = 'http://example.com';

          return spark.internal.team._inferTeamUrl(testTeam)
            .then((team) => {
              assert.called(spark.internal.feature.getFeature);
              assert.notCalled(spark.internal.device.getServiceUrl);
              assert.equal(team.url, 'http://example.com');
            });
        });
        it('builds URL if not defined', () => {
          delete testTeam.url;

          return spark.internal.team._inferTeamUrl(testTeam)
            .then((team) => {
              assert.called(spark.internal.feature.getFeature);
              assert.called(spark.internal.device.getServiceUrl);
              assert.equal(team.url, `${convoUrl}/teams/id1`);
            });
        });
      });
      describe('HA is enabled', () => {
        beforeEach(() => {
          spark.internal.feature.getFeature = sinon.stub().returns(Promise.resolve(true));
          testTeam.id = 'id1';
        });
        it('builds URL if already defined', () => {
          testTeam.url = 'https://example.com';

          return spark.internal.team._inferTeamUrl(testTeam)
            .then((team) => {
              assert.called(spark.internal.feature.getFeature);
              assert.called(spark.internal.device.getServiceUrl);
              assert.equal(team.url, `${convoUrl}/teams/id1`);
            });
        });
        it('builds URL if not defined', () => {
          delete testTeam.url;

          return spark.internal.team._inferTeamUrl(testTeam)
            .then((team) => {
              assert.called(spark.internal.feature.getFeature);
              assert.called(spark.internal.device.getServiceUrl);
              assert.equal(team.url, `${convoUrl}/teams/id1`);
            });
        });
      });
    });
  });
});
